-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '1.9.0'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '1.10.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION




    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION

                --ELIMINAR DATA HISTORICA 
                DELETE FROM [dbo].[HistoricEvaluationFormat]
                DELETE FROM [dbo].[HistoricGroup]
                DELETE FROM [dbo].[HistoricSubGroup]
                DELETE FROM [dbo].[HistoricAbility]
                DELETE FROM [dbo].[HistoricAbilityEvaluationFormat]
                DELETE FROM [dbo].[HistoricAbilityQuery]
                DELETE FROM [dbo].[HistoricAbilityQueryEntity]
                DELETE FROM [dbo].[HistoricEvaluationFormatCustomField]
                DELETE FROM [dbo].[HistoricCustomField]
                DELETE FROM [dbo].[HistoricEvaluationFormatAttachedSignature]
                DELETE FROM [dbo].[HistoricCustomFieldResult]
                DELETE FROM [dbo].[HistoricCampaignEvaluationFormat]
                DELETE FROM [dbo].[HistoricTemplate]
                DELETE FROM [dbo].[HistoricTemplateTheme]
                DELETE FROM [dbo].[HistoricTemplateThemeItem]
                DELETE FROM [dbo].[HistoricTemplateThemeItemEntity]
                

                --GUARDAR DATA HISTORICA PARA LOS PLANES ANTIGUOS
                DECLARE RECURRENT_CURSOR CURSOR 
                LOCAL STATIC READ_ONLY FORWARD_ONLY
                FOR SELECT [Instance],[ID],[IdEvaluationFormat] FROM [dbo].[EvaluationPlan] (NOLOCK) 

                -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
			    DECLARE @VCC    VARCHAR(50) = ''
                DECLARE @ID     VARCHAR(50) = ''
                DECLARE @FORMAT VARCHAR(50) = ''
                -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
                OPEN RECURRENT_CURSOR

                FETCH NEXT FROM RECURRENT_CURSOR INTO @VCC,@ID,@FORMAT
                WHILE @@FETCH_STATUS = 0
                BEGIN 

                    EXEC [dbo].[SaveHistoricEvaluationPlanData] @VCC,@ID,@FORMAT
                    
                
                    FETCH NEXT FROM RECURRENT_CURSOR INTO @VCC,@ID,@FORMAT
                END
                CLOSE RECURRENT_CURSOR
                DEALLOCATE RECURRENT_CURSOR



                --GUARDAR DATA HISTORICA PARA LOS ESTUDIOS ANTIGUOS
                DECLARE RECURRENT_CURSOR CURSOR 
                LOCAL STATIC READ_ONLY FORWARD_ONLY
                FOR SELECT [VCC],[Name],[TemplateId] FROM [dbo].[Study] (NOLOCK) 

                -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
				DECLARE @VCC_STUDY    VARCHAR(50) = ''
                DECLARE @ID_STUDY    VARCHAR(50) = ''
                DECLARE @TEMPLATE VARCHAR(50) = ''
                -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
                OPEN RECURRENT_CURSOR

                FETCH NEXT FROM RECURRENT_CURSOR INTO @VCC_STUDY,@ID_STUDY,@TEMPLATE
                WHILE @@FETCH_STATUS = 0
                BEGIN 
                
                    EXEC [dbo].[SaveHistoricStudyData] @VCC_STUDY,@ID_STUDY,@TEMPLATE

                    FETCH NEXT FROM RECURRENT_CURSOR INTO @VCC_STUDY,@ID_STUDY,@TEMPLATE
                END
                CLOSE RECURRENT_CURSOR
                DEALLOCATE RECURRENT_CURSOR


    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    

    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, 
            [IsRegistered] = 0,
            [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH



